/* sagittal.h --- Sagittal support for microabc */

/* This program is part of microabc (C) 2006-2008 Hudson Lacerda */

/* Copyright  (C)  2006-2008  Hudson F. M. Lacerda <hfml @ brfree.com.br> */

/* Author: Hudson F. M. Lacerda <hfml @ brfree.com.br> */
/* Maintainer: Hudson F. M. Lacerda <hfml @ brfree.com.br> */
/* URL: http://br.geocities.com/hfmlacerda/ */

/* This program is free software; you can redistribute it and/or modify */
/* it under the terms of the GNU General Public License as published by */
/* the Free Software Foundation; either version 2 of the License, or */
/* (at your option) any later version. */

/* This program is distributed in the hope that it will be useful, */
/* but WITHOUT ANY WARRANTY; without even the implied warranty of */
/* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the */
/* GNU General Public License for more details. */

/* You should have received a copy of the GNU General Public License */
/* along with this program; if not, write to the Free Software */
/* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA */

#ifndef _SAGITTAL__H_
#define _SAGITTAL__H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "sagdata.h"

/* math.h */
#ifdef CYGWIN /* long-double functions disabled for CYGWIN */
#ifndef powl
#define powl(x,y) pow((double)(x),(double)(y))
#endif
#ifndef log2l
#define log2l(x) (log((double)(x)) / log(2.0))
#endif
#ifndef exp2l
#define exp2l(x) pow(2.0,(double)(x))
#endif
#ifndef trunc
#define trunc(x) ((double)((long int)(x)))
#endif
#ifndef round
#define round(x) ((double)((long int) ((x) +  ((x)>0?.5:(-.5))  )))
#endif
#else /* not CYGWIN */
long double powl(long double x, long double y);
long double log2l(long double x);
long double exp2l(long double x);
double round(double);
double trunc(double);
#endif

/* difference (in cents) between 3/2 fifth and 12-ET fifth */
#define FIFDIF (1.9550008653874328956590034067630767822266)

#define REFNOMINAL 3 /* reference nominal for adjusting: 3 for D, 4 for A
		        See adjustnominals() and adjustpythagoras() */

#define MINSAGCODE (32)   /* minumum meaningful char code in Sagittal */
#define MAXSAGCODE (255)  /* maximum meaningful char code in Sagittal */
#define MAXSAGNAME (128)  /* maximum size of sagittal-pitch name string */
#define MAXSAGACCID (8)   /* maximum number of accidentals in a single pitch */
#define MAXSAGACCENTS (3) /* maximum number of accents at left or right (with '\0') */

#define SAGNATURAL (144) /* code of natural in Sagittal */
#define SAGSPACE    (32) /* code of space in Sagittal */

/* ASCII formats */
#define SHORTPURE    1
#define SHORTMIXED   2
#define LONGPURE     4
#define LONGMIXED    8
#define ALTERNATIVE 16

/* Character to indicate a non-visible accidental */
#define DONTSHOW ':'

#define SAGOCTAVEUP '`'
#define SAGOCTAVEDN ','
#define SAG_DEFAULT_OCT 5

/* sagittal array of structures (in sagdata.c) */
extern sagstruct sagittal[NSAGCHARS];

#define NPRIMES 11 /* number of elements of array primes[] */

/* sagittal accidental */
typedef struct {
    char name[MAXSAGNAME]; /* text representation */
    int code; /* code for the main accidental */
    char left[MAXSAGACCENTS]; /* left accents */
    char right[MAXSAGACCENTS]; /* right accents */
    short int visible;
} sagaccid;

/* sagittal pitch */
typedef struct {
    char name[MAXSAGNAME]; /* store sagittal pitch */
    char nominal; /* character for nominal of the pitch (like A B C...) */
    int octave; /* index of octave (or pseudo-octave) */
    sagaccid accidental; /* sagittal accidental (can have accents) */
} sagpitch;

/* interpret pitchname and store in the struct pitch */
int setsagpitch(sagpitch *pitch, const char name[],
		const char nominals[], const unsigned char asciiformat);
int copysagpitch(sagpitch *dest, sagpitch *orig);
int copysagaccid(sagaccid *dest, sagaccid *orig);
int cleansagpitch(sagpitch *sag);

/* Convert Sagittal pitch to microtonal ABC pitch */
/* pyth boolean option is to convert to pythagorean nominals.
   This requires that ALL NOTES MUST HAVE A SAGITTAL ACCIDENTAL. */
/* temper[0] is 2/1 tempering; temper[1] is 3/1 tempering (in cents) */
int sagcode2abc (char *dest, int code, char nominal,
		 int precision, int pyth /* boolean: ajust nominals or not */,
		 double temper[]);
double sagtemper(int code,double temper[]);
/* Get the power of base in fraction */
double primefactor(double num, double den, double base);

int sagasciiname2code(const char *a, const unsigned char f);

/* adjust Sagittal pitch into Pythagorean tuning (using =D as reference) */
int adjustpythagoras (double *num, int den, char nominal);
/* adjust Sagittal pitch into arbitrary temperament (using =D as reference) */
int adjustnominals (double *num, int den, char nominal, double temper[]);


#endif
